#include "floatdefs.hh"
#include "matrix.hh"
#include "xyz.hh"
#include "map.hh"
#include "rgb.hh"

#include "trace.hh"

const RGB Trace(Map *m, const XYZ &eye, const XYZ &dir)
{
	MapWall *w = m->FindObstacle(eye, dir);
	if(!w)
	{
		/* Nothing there */
		RGB tmp(0, 0, 0.3);
		return tmp;
	}
	
	XYZ hit = w->HitPoint(eye, dir);
	RGB tmp = w->Pigm;
	tmp /= hit.dist(eye);
	return tmp;
}

void Trace(unsigned xres, unsigned yres,
           const XYZ &eye,
           const Matrix &meitriks,
           FLOAT_ARG zoom,
           Map *m
          )
{
    static const unsigned char tgaBuf[] = {0,0,2, 0,0, 0,0, 0,  0,0, 0,0, xres&256, xres>>8, yres&255, yres>>8, 24,32};
    FILE *fp = fopen("diu.tga", "wb");
    fwrite(tgaBuf, 1, sizeof(tgaBuf), fp);
        
	for(unsigned y=0; y<yres; ++y)
	{
		const FLOAT ydisp = (y / (FLOAT)(yres-1) - 0.5);
		for(unsigned x=0; x<xres; ++x)
		{
			const FLOAT xdisp = (x / (FLOAT)(xres-1) - 0.5);
			
			XYZ dir(xdisp, ydisp, zoom);
			meitriks.Rotate(dir);
			
			RGB res = Trace(m, eye, dir);
			
			unsigned char R=res.getR(), G=res.getG(), B=res.getB();
			putc(B, fp);
			putc(G, fp);
			putc(R, fp);
		}
	}
	fclose(fp);
}
